@@ -254,45 +254,7 @@ class RealtimeStorm(Storm):
         full_path = os.path.join(save_path,f"Forecast_{self.id}.{url_ext}")
         with open(full_path, 'wb') as f:
             f.write(response.content)
-        
-    
-    def get_nhc_discussion_realtime(self):
-        
-        r"""
-        Retrieve the latest available forecast discussion. For JTWC storms, the Prognostic Reasoning product is retrieved.
-        
-        Returns
-        -------
-        dict
-            Dict entry containing the latest official forecast discussion.
-        """
-        
-        #Warn about pending deprecation
-        warnings.warn("'get_nhc_discussion_realtime' will be deprecated in future Tropycal versions, use 'get_discussion_realtime' instead",DeprecationWarning)
-        
-        #Check to ensure storm is not an invest
-        if self.invest:
-            raise RuntimeError("Error: NHC does not issue advisories for invests that have not been designated as Potential Tropical Cyclones.")
-        
-        #Get latest forecast discussion for HURDAT source storm objects
-        if self.source == "hurdat":
-            return self.get_nhc_discussion(forecast=-1)
-        
-        #Get latest forecast discussion for JTWC source storm objects
-        elif self.source == 'jtwc':
-            
-            #Read in discussion file
-            url = f"https://www.metoc.navy.mil/jtwc/products/{self.id[0:2].lower()}{self.id[2:4]}{self.id[6:8]}prog.txt"
-            f = urllib.request.urlopen(url)
-            content = f.read()
-            content = content.decode("utf-8")
-            f.close()
-            return content
-        
-        #Otherwise, return error message
-        else:
-            msg = "No realtime forecast discussion is available for this storm."
-            raise RuntimeError(msg)
+
     
     def get_discussion_realtime(self):
         
@@ -337,7 +299,7 @@ class RealtimeStorm(Storm):
         Parameters
         ----------
         ssl_certificate : boolean, optional
-            If a JTWC forecast, this determines whether to disable SSL certificate when retrieving data from the default JTWC source ("jtwc"). Default is True. Use False *ONLY* if True causes an SSL certification error.
+            If a JTWC forecast, this determines whether to disable SSL certificate when retrieving data from JTWC. Default is True. Use False *ONLY* if True causes an SSL certification error.
         
         Returns
         -------
@@ -373,12 +335,18 @@ class RealtimeStorm(Storm):
                 #Get basic components
                 lineArray = [i.replace(" ","") for i in line]
                 if len(lineArray) < 11: continue
-                basin,number,run_init,n_a,model,fhr,lat,lon,vmax,mslp,stype = lineArray[:11]
+                try:
+                    basin,number,run_init,n_a,model,fhr,lat,lon,vmax,mslp,stype,rad,windcode,neq,seq,swq,nwq = lineArray[:17]
+                    use_wind = True
+                except:
+                    basin,number,run_init,n_a,model,fhr,lat,lon,vmax,mslp,stype = lineArray[:11]
+                    use_wind = False
                 if model not in ["OFCL","OFCI"]: continue
 
                 if len(forecasts) == 0:
                     forecasts = {
-                        'fhr':[],'lat':[],'lon':[],'vmax':[],'mslp':[],'cumulative_ace':[],'cumulative_ace_fhr':[],'type':[],'init':dt.strptime(run_init,'%Y%m%d%H')
+                        'init':dt.strptime(run_init,'%Y%m%d%H'),'fhr':[],'lat':[],'lon':[],'vmax':[],'mslp':[],'type':[],
+                        'windrad':[],'cumulative_ace':[],'cumulative_ace_fhr':[]
                     }
 
                 #Format lat & lon
@@ -407,7 +375,29 @@ class RealtimeStorm(Storm):
                 else:
                     mslp = int(mslp)
                     if mslp < 1: mslp = np.nan
-
+                    
+                #Format wind radii
+                if use_wind:
+                    try:
+                        rad = int(rad)
+                        if rad in [0,35]: rad = 34
+                        neq = np.nan if windcode=='' else int(neq)
+                        seq = np.nan if windcode in ['','AAA'] else int(seq)
+                        swq = np.nan if windcode in ['','AAA'] else int(swq)
+                        nwq = np.nan if windcode in ['','AAA'] else int(nwq)
+                    except:
+                        rad = 34
+                        neq = np.nan
+                        seq = np.nan
+                        swq = np.nan
+                        nwq = np.nan
+                else:
+                    rad = 34
+                    neq = np.nan
+                    seq = np.nan
+                    swq = np.nan
+                    nwq = np.nan
+                
                 #Add forecast data to dict if forecast hour isn't already there
                 if fhr not in forecasts['fhr']:
                     if model in ['OFCL','OFCI'] and fhr > 120:
@@ -420,18 +410,25 @@ class RealtimeStorm(Storm):
                         forecasts['lon'].append(lon)
                         forecasts['vmax'].append(vmax)
                         forecasts['mslp'].append(mslp)
+                        forecasts['windrad'].append({rad:[neq,seq,swq,nwq]})
 
                         #Get storm type, if it can be determined
                         if stype in ['','DB'] and vmax != 0 and np.isnan(vmax) == False:
                             stype = get_storm_type(vmax,False)
                         forecasts['type'].append(stype)
-        
+                else:
+                    ifhr = forecasts['fhr'].index(fhr)
+                    forecasts['windrad'][ifhr][rad] = [neq,seq,swq,nwq]
+                    
         #Retrieve JTWC forecast otherwise
         else:
             
             #Get forecast for this storm
-            url = f"https://www.nrlmry.navy.mil/atcf_web/docs/current_storms/{self.id.lower()}.sum"
-            if ssl_certificate == False and self.source == 'jtwc':
+            if self.jtwc_source in ['jtwc','ucar']:
+                url = f"https://www.nrlmry.navy.mil/atcf_web/docs/current_storms/{self.id.lower()}.sum"
+            else:
+                url = f"https://www.ssd.noaa.gov/PS/TROP/DATA/ATCF/JTWC/{self.id.lower()}.fst"
+            if ssl_certificate == False and self.jtwc_source in ['jtwc','ucar']:
                 import ssl
                 if requests.get(url,verify=False).status_code != 200:
                     raise RuntimeError("JTWC forecast data is unavailable for this storm.")
@@ -439,7 +436,7 @@ class RealtimeStorm(Storm):
                 if requests.get(url).status_code != 200: raise RuntimeError("JTWC forecast data is unavailable for this storm.")
 
             #Read file content
-            if ssl_certificate == False and self.source == 'jtwc':
+            if ssl_certificate == False and self.jtwc_source in ['jtwc','ucar']:
                 import ssl
                 f = urllib.request.urlopen(url,context=ssl._create_unverified_context())
             else:
@@ -449,64 +446,174 @@ class RealtimeStorm(Storm):
             content = content.split("\n")
             f.close()
 
-            #Iterate through every line in content:
-            run_init = content[2].split(" ")[0]
-            forecasts = {}
+            if self.jtwc_source in ['jtwc','ucar']:
+                #Iterate through every line in content:
+                run_init = content[2].split(" ")[0]
+                forecasts = {}
 
-            for line in content[3:]:
-                
-                #Exit once done retrieving forecast
-                if line == "AMP": break
-                
-                #Get basic components
-                lineArray = line.split(" ")
-                if len(lineArray) < 4: continue
-                
-                #Exit once done retrieving forecast
-                if lineArray[0] == "AMP": break
-                    
-                if len(forecasts) == 0:
-                    forecasts = {
-                        'fhr':[],'lat':[],'lon':[],'vmax':[],'mslp':[],'cumulative_ace':[],'cumulative_ace_fhr':[],'type':[],'init':dt.strptime(run_init,'%Y%m%d%H')
-                    }
+                for line in content[3:]:
 
-                #Forecast hour
-                fhr = int(lineArray[0].split("T")[1])
-                
-                #Format lat & lon
-                lat = lineArray[1]
-                lon = lineArray[2]
-                if "N" in lat:
-                    lat_temp = lat.split("N")[0]
-                    lat = np.round(float(lat_temp) * 0.1,1)
-                elif "S" in lat:
-                    lat_temp = lat.split("S")[0]
-                    lat = np.round(float(lat_temp) * -0.1,1)
-                if "W" in lon:
-                    lon_temp = lon.split("W")[0]
-                    lon = np.round(float(lon_temp) * -0.1,1)
-                elif "E" in lon:
-                    lon_temp = lon.split("E")[0]
-                    lon = np.round(float(lon_temp) * 0.1,1)
+                    #Exit once done retrieving forecast
+                    if line == "AMP": break
 
-                #Format vmax & MSLP
-                vmax = int(lineArray[3])
-                if vmax < 10 or vmax > 300: vmax = np.nan
-                mslp = np.nan
+                    #Get basic components
+                    lineArray = line.split(" ")
+                    if len(lineArray) < 4: continue
 
-                #Add forecast data to dict if forecast hour isn't already there
-                if fhr not in forecasts['fhr']:
-                    if lat == 0.0 and lon == 0.0: continue
-                    forecasts['fhr'].append(fhr)
-                    forecasts['lat'].append(lat)
-                    forecasts['lon'].append(lon)
-                    forecasts['vmax'].append(vmax)
-                    forecasts['mslp'].append(mslp)
+                    #Exit once done retrieving forecast
+                    if lineArray[0] == "AMP": break
 
-                    #Get storm type, if it can be determined
-                    stype = get_storm_type(vmax,False)
-                    forecasts['type'].append(stype)
+                    if len(forecasts) == 0:
+                        forecasts = {
+                            'init':dt.strptime(run_init,'%Y%m%d%H'),'fhr':[],'lat':[],'lon':[],'vmax':[],'mslp':[],
+                            'windrad':[],'cumulative_ace':[],'cumulative_ace_fhr':[],'type':[]
+                        }
+
+                    #Forecast hour
+                    fhr = int(lineArray[0].split("T")[1])
+
+                    #Format lat & lon
+                    lat = lineArray[1]
+                    lon = lineArray[2]
+                    if "N" in lat:
+                        lat_temp = lat.split("N")[0]
+                        lat = np.round(float(lat_temp) * 0.1,1)
+                    elif "S" in lat:
+                        lat_temp = lat.split("S")[0]
+                        lat = np.round(float(lat_temp) * -0.1,1)
+                    if "W" in lon:
+                        lon_temp = lon.split("W")[0]
+                        lon = np.round(float(lon_temp) * -0.1,1)
+                    elif "E" in lon:
+                        lon_temp = lon.split("E")[0]
+                        lon = np.round(float(lon_temp) * 0.1,1)
+
+                    #Format vmax & MSLP
+                    vmax = int(lineArray[3])
+                    if vmax < 10 or vmax > 300: vmax = np.nan
+                    mslp = np.nan
+
+                    #Format wind radii
+                    windrad = {}
+                    for rad in (34,50,64):
+                        try:
+                            irad = list(lineArray).index(f'R{rad:03}')
+                            windrad[rad] = [int(lineArray[irad+j]) for j in (1,4,7,10)]
+                        except:
+                            continue
+
+                    #Add forecast data to dict if forecast hour isn't already there
+                    if fhr not in forecasts['fhr']:
+                        if lat == 0.0 and lon == 0.0: continue
+                        forecasts['fhr'].append(fhr)
+                        forecasts['lat'].append(lat)
+                        forecasts['lon'].append(lon)
+                        forecasts['vmax'].append(vmax)
+                        forecasts['mslp'].append(mslp)
+                        forecasts['windrad'].append(windrad)
+
+                        #Get storm type, if it can be determined
+                        stype = get_storm_type(vmax,False)
+                        forecasts['type'].append(stype)
             
+            else:
+                
+                content = [(i.replace(" ","")).split(",") for i in content]
+                
+                #Iterate through every line in content:
+                forecasts = {}
+
+                for line in content:
+
+                    #Get basic components
+                    lineArray = [i.replace(" ","") for i in line]
+                    if len(lineArray) < 11: continue
+                    try:
+                        basin,number,run_init,n_a,model,fhr,lat,lon,vmax,mslp,stype,rad,windcode,neq,seq,swq,nwq = lineArray[:17]
+                        use_wind = True
+                    except:
+                        basin,number,run_init,n_a,model,fhr,lat,lon,vmax,mslp,stype = lineArray[:11]
+                        use_wind = False
+                    if model not in ["JTWC"]: continue
+
+                    if len(forecasts) == 0:
+                        forecasts = {
+                            'init':dt.strptime(run_init,'%Y%m%d%H'),'fhr':[],'lat':[],'lon':[],'vmax':[],'mslp':[],'type':[],
+                            'windrad':[],'cumulative_ace':[],'cumulative_ace_fhr':[]
+                        }
+
+                    #Format lat & lon
+                    fhr = int(fhr)
+                    if "N" in lat:
+                        lat_temp = lat.split("N")[0]
+                        lat = np.round(float(lat_temp) * 0.1,1)
+                    elif "S" in lat:
+                        lat_temp = lat.split("S")[0]
+                        lat = np.round(float(lat_temp) * -0.1,1)
+                    if "W" in lon:
+                        lon_temp = lon.split("W")[0]
+                        lon = np.round(float(lon_temp) * -0.1,1)
+                    elif "E" in lon:
+                        lon_temp = lon.split("E")[0]
+                        lon = np.round(float(lon_temp) * 0.1,1)
+
+                    #Format vmax & MSLP
+                    if vmax == '':
+                        vmax = np.nan
+                    else:
+                        vmax = int(vmax)
+                        if vmax < 10 or vmax > 300: vmax = np.nan
+                    if mslp == '':
+                        mslp = np.nan
+                    else:
+                        mslp = int(mslp)
+                        if mslp < 1: mslp = np.nan
+
+                    #Format wind radii
+                    if use_wind:
+                        try:
+                            rad = int(rad)
+                            if rad in [0,35]: rad = 34
+                            neq = np.nan if windcode=='' else int(neq)
+                            seq = np.nan if windcode in ['','AAA'] else int(seq)
+                            swq = np.nan if windcode in ['','AAA'] else int(swq)
+                            nwq = np.nan if windcode in ['','AAA'] else int(nwq)
+                        except:
+                            rad = 34
+                            neq = np.nan
+                            seq = np.nan
+                            swq = np.nan
+                            nwq = np.nan
+                    else:
+                        rad = 34
+                        neq = np.nan
+                        seq = np.nan
+                        swq = np.nan
+                        nwq = np.nan
+
+                    #Add forecast data to dict if forecast hour isn't already there
+                    if fhr not in forecasts['fhr']:
+                        if model in ['OFCL','OFCI'] and fhr > 120:
+                            pass
+                        else:
+                            if lat == 0.0 and lon == 0.0:
+                                continue
+                            forecasts['fhr'].append(fhr)
+                            forecasts['lat'].append(lat)
+                            forecasts['lon'].append(lon)
+                            forecasts['vmax'].append(vmax)
+                            forecasts['mslp'].append(mslp)
+                            forecasts['windrad'].append({rad:[neq,seq,swq,nwq]})
+
+                            #Get storm type, if it can be determined
+                            if stype in ['','DB'] and vmax != 0 and np.isnan(vmax) == False:
+                                stype = get_storm_type(vmax,False)
+                            if stype == 'TY': stype = 'HU'
+                            forecasts['type'].append(stype)
+                    else:
+                        ifhr = forecasts['fhr'].index(fhr)
+                        forecasts['windrad'][ifhr][rad]=[neq,seq,swq,nwq]
+                    
         #Determine ACE thus far (prior to initial forecast hour)
         ace = 0.0
         for i in range(len(self.date)):
@@ -584,7 +691,7 @@ class RealtimeStorm(Storm):
         Other Parameters
         ----------------
         ssl_certificate : boolean, optional
-            If a JTWC forecast, this determines whether to disable SSL certificate when retrieving data from the default JTWC source ("jtwc"). Default is True. Use False *ONLY* if True causes an SSL certification error.
+            If a JTWC forecast, this determines whether to disable SSL certificate when retrieving data from JTWC. Default is True. Use False *ONLY* if True causes an SSL certification error.
         prop : dict
             Property of storm track lines.
         map_prop : dict
