@@ -15,6 +15,7 @@ import urllib
 import matplotlib.dates as mdates
 import matplotlib.colors as mcolors
 import matplotlib as mlib
+from matplotlib import path
 import warnings
 import scipy.interpolate as interp
 import re
@@ -143,8 +144,6 @@ def get_storm_classification(wind_speed,subtropical_flag,basin):
     
     These strings take the format of "Tropical Storm", "Hurricane", "Typhoon", etc.
     
-    Warning: This function currently does not differentiate between 1-minute, 3-minute and 10-minute sustained wind speeds.
-    
     Parameters
     ----------
     wind_speed : int
@@ -158,6 +157,14 @@ def get_storm_classification(wind_speed,subtropical_flag,basin):
     -------
     str
         String denoting the classification of the tropical cyclone.
+    
+    Notes
+    -----
+    
+    .. warning::
+
+        This function currently does not differentiate between 1-minute, 3-minute and 10-minute sustained wind speeds.
+    
     """
     
     #North Atlantic and East Pacific basins
@@ -263,6 +270,27 @@ def get_storm_type(wind_speed,subtropical_flag):
     -------
     str
         String denoting the tropical cyclone type.
+    
+    Notes
+    -----
+    The available types and their descriptions are as follows:
+    
+    .. list-table:: 
+       :widths: 25 75
+       :header-rows: 1
+
+       * - Property
+         - Description
+       * - TD
+         - Tropical Depression
+       * - TS
+         - Tropical Storm
+       * - HU
+         - Hurricane
+       * - SD
+         - Subtropical Depression
+       * - SS
+         - Subtropical Storm
     """
     
     #Tropical depression
@@ -283,7 +311,7 @@ def get_storm_type(wind_speed,subtropical_flag):
     else:
         return "HU"
 
-def get_basin(lat,lon,storm_id=""):
+def get_basin(lat,lon,source_basin=""):
     
     r"""
     Returns the current basin of the tropical cyclone.
@@ -297,20 +325,24 @@ def get_basin(lat,lon,storm_id=""):
     
     Other Parameters
     ----------------
-    storm_id : str
-        String representing storm ID. Used to distinguish between Atlantic and Pacific basins.
+    source_basin : str, optional
+        String representing the origin storm basin (e.g., "north_atlantic", "east_pacific").
     
     Returns
     -------
     str
         String representing the current basin (e.g., "north_atlantic", "east_pacific").
+    
+    Notes
+    -----
+    For storms in the North Atlantic or East Pacific basin, ``source_basin`` must be provided. This is because storms located over Mexico or Central America could be in either basin depending on where they originated (e.g., storms originated in the Atlantic basin are considered to be within the Atlantic basin while over Mexico or Central America until emerging in the Pacific Ocean).
     """
     
     #Error check
-    if isinstance(lat,float) == False and isinstance(lat,int) == False:
+    if isinstance(lat,(int,np.int,np.integer,float,np.floating)) == False:
         msg = "\"lat\" must be of type int or float."
         raise TypeError(msg)
-    if isinstance(lon,float) == False and isinstance(lon,int) == False:
+    if isinstance(lon,(int,np.int,np.integer,float,np.floating)) == False:
         msg = "\"lon\" must be of type int or float."
         raise TypeError(msg)
     
@@ -321,17 +353,29 @@ def get_basin(lat,lon,storm_id=""):
     if lat >= 0.0:
         
         if lon < 100.0:
-            return "north_indian"
-        elif lon < 180.0:
+            if lat < 40.0:
+                return "north_indian"
+            else:
+                if lon < 70.0:
+                    return "north_atlantic"
+                else:
+                    return "west_pacific"
+        elif lon <= 180.0:
             return "west_pacific"
         else:
-            if len(storm_id) != 8:
+            if source_basin == "north_atlantic":
+                if constants.PATH_PACIFIC.contains_point((lat,lon)):
+                    return "east_pacific"
+                else:
+                    return "north_atlantic"
+            elif source_basin == "east_pacific":
+                if constants.PATH_ATLANTIC.contains_point((lat,lon)):
+                    return "north_atlantic"
+                else:
+                    return "east_pacific"
+            else:
                 msg = "Cannot determine whether storm is in North Atlantic or East Pacific basins."
                 raise RuntimeError(msg)
-            if storm_id[0:2] == "AL":
-                return "north_atlantic"
-            else:
-                return "east_pacific"
     
     #Southern hemisphere check
     else:
@@ -384,7 +428,7 @@ def accumulated_cyclone_energy(wind_speed,hours=6):
     
     Parameters
     ----------
-    wind_speed : int
+    wind_speed : int or list, numpy.ndarray
         Sustained wind in knots.
     hours : int, optional
         Duration in hours over which the sustained wind was observed. Default is 6 hours.
@@ -393,16 +437,33 @@ def accumulated_cyclone_energy(wind_speed,hours=6):
     -------
     float
         Accumulated cyclone energy.
+    
+    Notes
+    -----
+    
+    As defined in `Bell et al. (2000)`_, Accumulated Cyclone Energy (ACE) is calculated as follows:
+    
+    .. math:: ACE = 10^{-4} \sum v^{2}_{max}
+    
+    As shown above, ACE is the sum of the squares of the estimated maximum sustained wind speed (in knots). By default, this assumes data is provided every 6 hours, as is the standard in HURDATv2 and NHC's Best Track, though this function provides an option to use a different hour duration.
+    
+    .. _Bell et al. (2000): https://journals.ametsoc.org/view/journals/bams/81/6/1520-0477_2000_81_s1_caf_2_0_co_2.xml
     """
     
+    #Determine types
+    if isinstance(wind_speed,(np.ndarray,list)):
+        wind_speed = np.copy(wind_speed)
+    
     #Calculate ACE
     ace = ((10**-4) * (wind_speed**2)) * (hours/6.0)
     
     #Coerce to zero if wind speed less than TS force
-    if wind_speed < 34: ace = 0.0
-    
-    #Return ACE
-    return ace
+    if isinstance(ace,(np.ndarray,list)):
+        ace[wind_speed < 34] = 0.0
+        return np.round(ace,4)
+    else:
+        if wind_speed < 34: ace = 0.0
+        return round(ace,4)
 
 def dropsonde_mslp_estimate(mslp,surface_wind):
     
@@ -890,6 +951,73 @@ def generate_nhc_cone(forecast,basin,shift_lons=False,cone_days=5,cone_year=None
                    'center_lon':interp_lon,'center_lat':interp_lat,'year':cone_year}
         return return_dict
 
+def calc_ensemble_ellipse(member_lons,member_lats):
+    
+    r"""
+    Calculate an ellipse representing ensemble member location spread. This function follows the methodology of Hamill et al. (2011).
+    
+    Parameters
+    ----------
+    member_lons : list
+        List containing longitudes of ensemble members valid at a single time.
+    member_lats : list
+        List containing latitudes of ensemble members valid at a single time.
+    
+    Returns
+    -------
+    dict
+        Dictionary containing the longitude and latitude of the ellipse.
+    
+    Notes
+    -----
+    The ensemble ellipse used in this function follows the methodology of `Hamill et al. (2011)`_, denoting the spread in ensemble member cyclone positions. The size of the ellipse is calculated to contain 90% of ensemble members at any given time. This ellipse can be used to determine the primary type of ensemble variability:
+
+    * **Along-track variability** - if the major axis of the ellipse is parallel to the ensemble mean motion vector.
+    * **Across-track variability** - if the major axis of the ellipse is normal to the ensemble mean motion vector.
+
+    .. _Hamill et al. (2011): https://doi.org/10.1175/2010MWR3456.1
+    
+    This code is adapted from NCAR Command Language (NCL) code courtesy of Ryan Torn and Thomas Hamill.
+    """
+    
+    #Compute ensemble mean lon & lat
+    mean_lon = np.average(member_lons)
+    mean_lat = np.average(member_lats)
+
+    Pb = [[0,0],[0,0]]
+    for i in range(len(member_lats)):
+        Pb[0][0] = Pb[0][0] + (member_lons[i]-mean_lon) * (member_lons[i]-mean_lon)
+        Pb[1][1] = Pb[1][1] + (member_lats[i]-mean_lat) * (member_lats[i]-mean_lat)
+        Pb[1][0] = Pb[1][0] + (member_lats[i]-mean_lat) * (member_lons[i]-mean_lon)
+    Pb[0][1] = Pb[1][0]
+    Pb = np.array(Pb) / float(len(member_lats)-1)
+
+    rho = Pb[1][0] / (np.sqrt(Pb[0][0]) * np.sqrt(Pb[1][1]))
+    sigmax = np.sqrt(Pb[0][0])
+    sigmay = np.sqrt(Pb[1][1])
+    fac = 1.0 / (2.0 * (1 - rho**2))
+
+    #Calculate lon & lat coordinates of ellipse
+    ellipse_lon = []
+    ellipse_lat = []
+    increment = np.pi/180.0
+    for radians in np.arange(0,(2.0*np.pi)+increment,increment):
+        xstart = np.cos(radians)
+        ystart = np.sin(radians)
+
+        for rdistance in np.arange(0.,2400.):
+            xloc = xstart * rdistance/80.0
+            yloc = ystart * rdistance/80.0
+            dist = np.sqrt(xloc * xloc + yloc * yloc)
+            prob = np.exp(-1.0 * fac * ((xloc/sigmax)**2 + (yloc/sigmay)**2 - 2.0 * rho * (xloc/sigmax)*(yloc/sigmay)))
+            if prob < 0.256:
+                ellipse_lon.append(xloc + mean_lon)
+                ellipse_lat.append(yloc + mean_lat)
+                break
+
+    #Return ellipse data
+    return {'ellipse_lon':ellipse_lon, 'ellipse_lat':ellipse_lat}
+
 #===========================================================================================================
 # Private utilities
 # These are primarily intended to be used internally. Do not add these to documentation.
@@ -955,7 +1083,7 @@ def category_label_to_wind(category):
     else:
         return category_to_wind(5)
 
-def dynamic_map_extent(min_lon,max_lon,min_lat,max_lat):
+def dynamic_map_extent(min_lon,max_lon,min_lat,max_lat,recon=False):
 
     r"""
     Sets up a dynamic map extent with an aspect ratio of 3:2 given latitude and longitude bounds.
@@ -970,6 +1098,8 @@ def dynamic_map_extent(min_lon,max_lon,min_lat,max_lat):
         Minimum latitude bound.
     max_lat : float
         Maximum latitude bound.
+    recon : bool
+        Zoom in plots closer for recon plots.
 
     Returns:
     --------
@@ -1025,6 +1155,7 @@ def dynamic_map_extent(min_lon,max_lon,min_lat,max_lat):
         factor = 0.75
     if min(xrng,yrng) < 6.0:
         factor = 0.9
+    if recon: factor = factor * 0.3
     bound_w = bound_w-(xrng*factor)
     bound_e = bound_e+(xrng*factor)
     bound_s = bound_s-(yrng*factor)
